@@ -53,7 +53,7 @@ alsa_object_open(struct audio_object *object,
 		return -EEXIST;
 
 	snd_pcm_format_t pcm_format;
-#define FORMAT(srcfmt, dstfmt, size) case srcfmt: pcm_format = dstfmt; self->sample_size = size; break;
+#define FORMAT(srcfmt, dstfmt, size) case srcfmt: pcm_format = dstfmt; self->sample_size = size*channels; break;
 	switch (format)
 	{
 	FORMAT(AUDIO_OBJECT_FORMAT_ALAW,      SND_PCM_FORMAT_A_LAW, 1)
@@ -99,6 +99,8 @@ alsa_object_open(struct audio_object *object,
 
 	snd_pcm_hw_params_t *params = NULL;
 	snd_pcm_hw_params_malloc(&params);
+	unsigned int period_time = LATENCY * 1000;
+	int dir = 0;
 
 	int err = 0;
 	if ((err = snd_pcm_open(&self->handle, self->device ? self->device : "default", SND_PCM_STREAM_PLAYBACK, 0)) < 0)
@@ -113,6 +115,8 @@ alsa_object_open(struct audio_object *object,
 		goto error;
 	if ((err = snd_pcm_hw_params_set_channels(self->handle, params, channels)) < 0)
 		goto error;
+	if ((err = snd_pcm_hw_params_set_period_time_near(self->handle, params, &period_time, &dir)) < 0)
+		goto error;
 	if ((err = snd_pcm_hw_params(self->handle, params)) < 0)
 		goto error;
 	if ((err = snd_pcm_prepare(self->handle)) < 0)
@@ -204,12 +208,18 @@ alsa_object_write(struct audio_object *object,
 			nToWrite -= nWritten;
 			data += nWritten * self->sample_size;
 			// Open question: if a signal caused the short read, should we snd_pcm_prepare?
-		} else if ((nWritten == -EPIPE) || (nWritten == -EBADFD)) {
+		} else if ((nWritten == -EPIPE)
+#ifdef EBADFD
+		    || (nWritten == -EBADFD)
+#endif
+		    ) {
 			// Either there was an underrun or the PCM was in a bad state.
 			err = snd_pcm_prepare(self->handle);
 			if (err != 0)
 				break;
-		} else if (nWritten == -ESTRPIPE) {
+		}
+#ifdef ESTRPIPE
+		else if (nWritten == -ESTRPIPE) {
 			// Sound suspended, try to resume.
 			do {
 				err = snd_pcm_resume(self->handle);
@@ -223,7 +233,9 @@ alsa_object_write(struct audio_object *object,
 			if (err < 0) {
 				break;
 			}
-		} else {
+		}
+#endif
+		else {
 			err = nWritten;
 			break;
 		}
